unit uPageControl;

interface

uses
{$IFDEF VCL}
  ComCtrls,
{$ELSE}
  QComCtrls,
{$ENDIF}
  Rubies;

var
  cPageControl: Tvalue;

function ap_cPageControl: Tvalue;
function ap_iPageControl(real: TPageControl; owner: Tvalue): Tvalue;
procedure Init_PageControl;

implementation

uses
  SysUtils, Classes,
  uDefUtils, uIntern, uHandle, uAlloc, uProp, uPhi,
  uSizeConstraints,
  uPersistent, uComponent, uControl;

function ap_cPageControl: Tvalue;
begin
  result := cPageControl;
end;

procedure PageControl_setup(obj: Tvalue; real: TPageControl);
begin
  ap_set_child_attr_module(obj);
//    AssignPropMethod(real, [Handle]);
end;

function PageControl_event_handle(This, name: Tvalue): Tvalue; cdecl;
begin
  EventHandle(This, name, [Handle]);
  result := Qnil;
end;

function PageControl_alloc(This: Tvalue; real: TPageControl): Tvalue;
begin
  result := ChildAlloc(This, real);
  PageControl_setup(result, real);
end;

function ap_iPageControl(real: TPageControl; owner: Tvalue): Tvalue;
begin
  result := PageControl_alloc(cPageControl, real);
  ap_owner(result, owner);
end;

function ap_iPageControl_v(var AControl; owner: Tvalue): Tvalue;
begin
  result := ap_iPageControl(TPageControl(AControl), owner);
end;

function PageControl_new(argc: integer; argv: Pointer; This: Tvalue): Tvalue; cdecl;
var
  real: TPageControl;
begin
  real := TPageControl.Create(nil);
  result := CompoAlloc(This, real);
  CompoSetup(argc, argv, real);
  PageControl_setup(result, real);
  ap_obj_call_init(result, argc, argv);
end;

function PageControl_select_next_page(argc: integer; argv: Pointer; This: Tvalue): Tvalue; cdecl;
var
  real: TPageControl;
  GoForward: Boolean;
begin
  if argc > 1 then ap_raise(ap_eArgError, sWrong_num_of_args);
  if argc = 1 then
    GoForward := RTEST(Pvalue(argv)^)
  else
    GoForward := True;
  real := ap_data_get_struct(This);
  real.SelectNextPage(GoForward);
  result := This;
end;

function PageControl_find_next_page(argc: integer; argv: Pointer; This: Tvalue): Tvalue; cdecl;
var
  args: array of Tvalue;
  real: TPageControl;
  CurPage: TTabSheet;
  GoForward: Boolean;
  CheckTabVisible: Boolean;
begin
  if argc < 1 then ap_raise(ap_eArgError, sToo_few_args);
  if argc > 3 then ap_raise(ap_eArgError, sWrong_num_of_args);
  SetLength(args, argc);
  args := argv;
  ap_data_get_object(args[0], TTabSheet, CurPage);
  if argc > 1 then
    GoForward := RTEST(args[1])
  else
    GoForward := True;
  if argc > 2 then
    CheckTabVisible := RTEST(args[2])
  else
    CheckTabVisible := True;
  real := ap_data_get_struct(This);
  real.FindNextPage(CurPage, GoForward, CheckTabVisible);
  result := This;
end;

function PageControl_pages(This, i: Tvalue): Tvalue; cdecl;
var
  real: TPageControl;
  n: Integer;
begin
  real := ap_data_get_struct(This);
  n := FIX2INT(i);
  result := This;
  try
    result := real.Pages[n].tag;
  except
    on E: Exception do
      ap_raise(ap_eArgError, PChar(E.message));
  end;
end;

function PageControl_get_page_count(This: Tvalue): Tvalue; cdecl;
var
  real: TPageControl;
begin
  real := ap_data_get_struct(This);
  result := INT2FIX(real.PageCount);
end;

procedure Init_PageControl;
begin
  OutputConstSetType(mPhi, TypeInfo(TTabStyle));
{$IFDEF VCL}
  OutputConstSetType(mPhi, TypeInfo(TTabPosition));
{$ENDIF}
  cPageControl := OutputPersistentClass(mPhi, TPageControl, cWinControl, ap_iPageControl_v);
  rb_define_method(cPageControl, 'event_handle', @PageControl_event_handle, 1);
  DefineSingletonMethod(cPageControl, 'new', PageControl_new);
  DefineMethod(cPageControl, 'select_next_page', PageControl_select_next_page);
  DefineMethod(cPageControl, 'find_next_page', PageControl_find_next_page);
  rb_define_method(cPageControl, 'pages', @PageControl_pages, 1);
  DefineAttrGet(cPageControl, 'page_count', PageControl_get_page_count);
end;

end.
